import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;

public class DebugData {

    /**checks that our data for the QI rational maps agrees with the
       numerically computed values.*/

    public static void test0() {
	System.out.println("DataMap data check");
	double[] x={Math.random(),Math.random()};
	for(int k=0;k<2;++k) {
	  for(int i=0;i<2;++i) {
	    for(int j=0;j<5;++j) {
		int[] a={i,j,k,0};
		double[] y={0,0};
		y=HeatSemigroup.main(a,x);
		QI2RationalMap F=new QI2RationalMap();  
		if(k==1) F=DataMap.map(a);
		if(k==0) F=DataGauss.map(a);
		double z0=F.F[0].evaluate(x[0],x[1]);
		double z1=F.F[1].evaluate(x[0],x[1]);
		System.out.println("compare "+i+" "+j+" "+k);
		System.out.println(y[0]+" "+y[1]);
		System.out.println(z0+" "+z1);
	    }
	  }
	}
    }


    /**This checks that the numerically computed differential
       is close to the algebraically computed differential*/

    public static void test1() {
	QI2Polynomial.SIZE=32;
	System.out.println("see derivative");
	int[] a={0,0,1,0};
	QI2RationalMap X=DataMap.map(a);
	QI2RationalFunction[][] DX=QI2Differential.D(X);
	double[] x={Math.random(),Math.random()};

	double[][] m=DebugDifferential.diffMap(a,x,.0000001); 
	System.out.println("compare derivative");
	for(int i=0;i<2;++i) {
	    for(int j=0;j<2;++j) {
		System.out.println(m[i][j]+" "+DX[i][j].evaluate(x[0],x[1]));
	    }
	}

	System.out.println("check Jacobian");

	QI2RationalFunction AA=QI2Differential.detD(X);
	double y1=AA.evaluate(x[0],x[1]);
	double y2=m[0][0]*m[1][1]-m[0][1]*m[1][0];
	System.out.println("compare Jacobians "+y1+" "+y2);

	System.out.println("check square norm calculation");
	QI2RationalFunction C1=QI2Differential.sumOfSquares1D(X);
	QI2RationalFunction C2=QI2Differential.sumOfSquares2D(X);
	double a1=C1.evaluate(x[0],x[1]);
	double a2=C2.evaluate(x[0],x[1]);

	double norm1=0;
	norm1=norm1+m[0][0]*m[0][0]+m[0][1]*m[0][1];
	double norm2=0;
	norm2=norm2+m[1][0]*m[1][0]+m[1][1]*m[1][1];

	System.out.println("compare "+norm1+" "+a1);
	System.out.println("compare "+norm2+" "+a2);

	System.out.println("check linear combo method");
        double v0=2*m[0][0]+m[0][1];
	double v1=2*m[1][0]+m[1][1];
	QI2RationalFunction[] V=QI2Differential.DIFF(X,new QI2(2,0),new QI2(1,0));
	double vv0=V[0].evaluate(x[0],x[1]);
	double vv1=V[1].evaluate(x[0],x[1]);
	System.out.println("compare "+v0+" "+vv0);
	System.out.println("compare "+v1+" "+vv1);

    }



    /**Tests that the heat map blows up at the special point.*/
    public static void test2() {
	System.out.println("check heat map blowup point");
	QI2[] X=DataPartition.point56();
	X[0].println();
	QI2RationalMap F=DataMap.map(0,0);
	QI2 p0=F.F[0].P.evaluate(X[0],X[1]);
	QI2 q0=F.F[0].Q.evaluate(X[0],X[1]);
	QI2 p1=F.F[1].P.evaluate(X[0],X[1]);
	QI2 q1=F.F[1].Q.evaluate(X[0],X[1]);
	p0.println();
	q0.println();
	p1.println();
	q1.println();
    }


    /**This tests the value of a certain map on a certain point*/

    public static void test3() {
	System.out.println("special test"); 
	int[] a={0,1,1,0};
	QI2RationalMap F=DataMap.map(a);
	QI2[] x=specialPoint();
	QI2 p0=F.F[0].P.evaluate(x[0],x[1]);
	QI2 q0=F.F[0].Q.evaluate(x[0],x[1]);
	QI2 p1=F.F[1].P.evaluate(x[0],x[1]);
	QI2 q1=F.F[1].Q.evaluate(x[0],x[1]);
	p0.println();
	q0.println();
	p1.println();
	q1.println();
    }

    public static QI2[] specialPoint() {
	QI a=new QI(0,1);
	QI b=new QI(0,0);
	QI c=new QI(1,-1);
	QI d=new QI(0,0);
	QI2 x=new QI2(a,b,1);
	QI2 y=new QI2(c,d,1);
	QI2[] X={x,y};
	return(X);
    }
}

