import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


public class QI2RationalFunction {
    QI2Polynomial P,Q;

    public QI2RationalFunction() {}

    public QI2RationalFunction(QI2 r) {
	this.P=new QI2Polynomial(r);
	this.Q=new QI2Polynomial(QI2.one());
    }


    public double evaluate(double x,double y) {
	double p=this.P.evaluate(x,y);
	double q=this.Q.evaluate(x,y);	
        double r=1.0*p/q;
	return(r);
    }


    public static QI2RationalFunction plus(QI2RationalFunction X,QI2RationalFunction Y) {
	QI2RationalFunction Z=new QI2RationalFunction();
	Z.P=QI2Polynomial.plus(QI2Polynomial.times(X.P,Y.Q),QI2Polynomial.times(X.Q,Y.P));
	Z.Q=QI2Polynomial.times(X.Q,Y.Q);
	return(Z);
    }


    public static QI2RationalFunction minus(QI2RationalFunction X,QI2RationalFunction Y) {
	QI2RationalFunction Z=new QI2RationalFunction();
	Z.P=QI2Polynomial.minus(QI2Polynomial.times(X.P,Y.Q),QI2Polynomial.times(X.Q,Y.P));
	Z.Q=QI2Polynomial.times(X.Q,Y.Q);
	return(Z);
    }

    public static QI2RationalFunction times(QI2RationalFunction X,QI2RationalFunction Y) {
	QI2RationalFunction Z=new QI2RationalFunction();
	Z.P=QI2Polynomial.times(X.P,Y.P);
	Z.Q=QI2Polynomial.times(X.Q,Y.Q);
	return(Z);
    }

    public QI2RationalFunction scale(QI2 r) {
	QI2RationalFunction Z=new QI2RationalFunction();
	Z.P=this.P.scale(r);
	Z.Q=this.Q;
	return(Z);
    }

    /**99 here represents either a pole or a fail state*/
    public int evaluateSign(QI2[] A) {
	QI2 PP=P.evaluate(A);
	QI2 QQ=Q.evaluate(A);
	int sp=PP.sign();
	int sq=QQ.sign();
	if(sp==99) return(99);
	if(sq==99) return(99);
	if(sq==0) return(99);
	if(sp==sq) return(1);
	return(-1);
    }

    /**Normalizes the sign of the denominator so that the
       first nonzero term is positive.*/

    public QI2RationalFunction fixSign() {
	int s=Q.firstTermSign();
	if(s==1) return(this);
	QI2RationalFunction F=new QI2RationalFunction();
	F.P=P.negate();
	F.Q=Q.negate();
	return(F);
    }

    /**multiplies the numerator and denominator (the same) by +/-1
       so that the denominator is non-negative on the given point.*/

    public QI2RationalFunction fixSign(QI2[] X) {
	QI2 Y=this.Q.evaluate(X);
	if(Y.isPositive()==true) return(this);
	QI2RationalFunction Z=new QI2RationalFunction();
	Z.P=this.P.negate();
	Z.Q=this.Q.negate();
	return(Z);
    }


    /**negation*/

    public QI2RationalFunction negate() {
	QI2RationalFunction F=new QI2RationalFunction();
	F.P=P.negate();
	F.Q=new QI2Polynomial(Q);
	return(F);
    }


    /**multiplies the numerator by +/-1 to make the
       function non-negative on the given point*/


    public QI2RationalFunction makePositive(QI2[] X) {
	QI2 p=this.P.evaluate(X);
	QI2 q=this.Q.evaluate(X);
	boolean pp=p.isPositive();
	boolean qq=q.isPositive();
	if((pp==false)&&(qq==false)) return(this);
	if((pp==true)&&(qq==true)) return(this);
	QI2RationalFunction Z=new QI2RationalFunction();
	Z.P=this.P.negate();
	Z.Q=new QI2Polynomial(this.Q);
	return(Z);
    }

    /**Both numerator and denominator have to be
       positive at the given point*/

    public boolean purePositive(QI2[] X) {
	QI2 p=this.P.evaluate(X);
	QI2 q=this.Q.evaluate(X);
	boolean pp=p.isPositive();
	boolean qq=q.isPositive();
	if((pp==true)&&(qq==true)) return true;
	return(false);
    }

}





        