import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;

/**This window appears at the bottom of the screen.  It has controls for some
   of the rendering aspects of the picture.*/

public class StyleCanvas extends ScaleCanvas implements MouseListener,MouseMotionListener,KeyListener {
    Manager M;
    Lever ZOOM,SCHEME,DENS,WIDTH,BOLD,SCALE,AFFINE,POLY,STYLE;
    ListenSquare EXPORT;
    SelectColor SC;  
    ListenSquare[] COL=new ListenSquare[14];


     public StyleCanvas() {
	 addMouseListener(this);
	 addMouseMotionListener(this);
	 addKeyListener(this);
	 int mem=0;

	 mem=PlotStyle.BOLD; 
         BOLD=new Lever(5,95,mem,5);     //color scheme choice for the Julia set  

	 mem=PlotStyle.STYLE;
	 STYLE=new Lever(300,95,mem,3); //Julia set rendering style

	
	 mem=PlotStyle.WIDTH;
         WIDTH=new Lever(5,20,mem,4);	 //width of the lines on the partition images

	 mem=PlotStyle.DENS;
         DENS=new Lever(285,20,mem,9);    //plot density for the partition images
        
         mem=SpecialAffine.getVal();
         AFFINE=new Lever(145,20,mem,2);   //toggles the canonical affine transorm

	 mem=PlotStyle.ZOOM;
         ZOOM=new Lever(145,55,mem,9);	 //controls the scale factor in the zooming

         SCALE=new Lever(285,55,2,4);    //gives 5 preset scale values	

 	 SC=new SelectColor(0,122,409,50); //color selector

	 int xpos=115;
	 int ypos=78;
	 for(int i=0;i<14;++i) {
              COL[i]=new ListenSquare(xpos,ypos,20,18);
	      xpos=xpos+20;
	      if(i==6) {
		  xpos=xpos-140;
		  ypos=ypos+20;
	      }
	 }

	 EXPORT=new ListenSquare(5,45,100,20); //export option

     }

    public void setColors(int d) {
	if(d==0) setColors0();
	if(d==1) setColors1();
	if(d==2) setColors2();
	if(d==3) setColors3();
	if(d==4) setColors4();
    }



    public void setColors0() {
	for(int i=0;i<7;++i) {
	    RenderJulia.COL[i]=new Color(255-16*i,255-16*i,255-16*i);
	}
	for(int i=0;i<7;++i) {
	    RenderJulia.COL[7+i]=new Color(255-16*i,255-16*i,255-16*i);
	}
    }

    public void setColors1() {
	for(int i=0;i<7;++i) {
	    RenderJulia.COL[i]=new Color(100,255-10*i,255-10*i);
	}
	for(int i=0;i<7;++i) {
	    RenderJulia.COL[7+i]=new Color(0,180-10*i,255-10*i);
	}
    }


    public void setColors2() { 

 Color[] COL={
    new Color(255,255,50),
    new Color(255,220,0),
    new Color(255,180,0),
    new Color(255,130,0),
    new Color(255,40,0),
    new Color(180,0,0),
    new Color(130,0,0), 
    new Color(50,255,255),
    new Color(0,220,255),
    new Color(0,180,255),
    new Color(0,120,255),
    new Color(0,50,255),
    new Color(0,0,220),
    new Color(0,0,190)};
     for(int i=0;i<14;++i) RenderJulia.COL[i]=COL[i];
    }

    public void setColors3() {

 Color[] COL={
    new Color(255,255,50),
    new Color(255,220,0),
    new Color(255,180,0),
    new Color(255,130,0),
    new Color(255,40,0),
    new Color(180,0,0),
    new Color(130,0,0), 
    new Color(50,255,255),
    new Color(0,220,255),
    new Color(0,180,255),
    new Color(0,120,255),
    new Color(0,50,255),
    new Color(0,0,220),
    new Color(0,0,190)};
      for(int i=0;i<14;++i) RenderJulia.COL[i]=COL[13-i];
    }

    public void setColors4() {
	for(int i=0;i<14;++i) RenderJulia.COL[i]=random();
    }



    public Color random() {
	int r=(int)(255*Math.random());
	int g=(int)(255*Math.random());
	int b=(int)(255*Math.random());
	return new Color(r,g,b);
    }




   public void paint(Graphics g2) {
      Graphics2D g=(Graphics2D) g2;
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      drawControls(g);
   }

    public void drawControls(Graphics2D g) {
	g.setColor(new Color(80,100,120));
	g.fillRect(0,0,getWidth(),getHeight());
	g.setColor(Color.white);
	g.drawRect(0,0,getWidth()-1,getHeight()-1);
	g.setColor(Color.white);
        g.setFont(new Font("Helvetica",Font.PLAIN,12));
	ZOOM.render(g,"zoom strength",Color.red);
	DENS.render(g,"polygon plot detail",Color.blue);
	WIDTH.render(g,"polygon outline",Color.blue);
	BOLD.render(g,"color schemes",Color.red);
	STYLE.render(g,"render style",Color.red);
	SCALE.render(g,"push for preset scale",Color.red);
	AFFINE.render(g,"special coord change (n/y)",Color.red);
	EXPORT.render(g,Color.blue);
	g.setColor(Color.white);
	g.drawString("export Julia set",10,60);
	SC.render(g);
	for(int i=0;i<14;++i) COL[i].render(g,RenderJulia.COL[i]);
    }

    public void setScaling(Point X) {
	if(ZOOM.isUsed(X)!=-1) {
	  int n=ZOOM.val-1;
	  double phi=(1+Math.sqrt(5))/2;
	  double d=Math.pow(phi,5)/2;
	  double e=Math.pow(2,-n);
	  M.P.SCALE=Math.pow(d,e);
	  PlotStyle.ZOOM=ZOOM.val;
	}
    }

    public void presetScales(Point X) {
	int a=AFFINE.val;
	int d=SCALE.isUsed(X);
	if(a==0) {
	  if(d==0) M.P.setScales(300,300,2000);
	  if(d==1) M.P.setScales(100,380,250);
	  if(d==2) M.P.setScales(188,408,20);
	  if(d==3) M.P.setScales(270,300,3);
	}
 	if(a==1) {
	  if(d==0) M.P.setScales(300,300,2000);
	  if(d==1) M.P.setScales(300,300,250);
	  if(d==2) M.P.setScales(150,300,25);
	  if(d==3) M.P.setScales(250,300,10);
	}
    }

    public void setSpecialAffine(Point X) {
	int d=AFFINE.isUsed(X);
	if(d==0) SpecialAffine.GO=false;
	if(d==1) SpecialAffine.GO=true;
    }

    public void mouseClicked(MouseEvent e) {
        MouseData J=MouseData.process(e);
        int d=ZOOM.process(J.X);
  	if(d==99) DocumentImage.zoomInfo(M);
        d=DENS.process(J.X);
	PlotStyle.DENS=DENS.val;
	if(d==99) DocumentImage.quadPlotInfo(M);
        d=WIDTH.process(J.X);
	PlotStyle.WIDTH=WIDTH.val;	
        if(d==99) DocumentImage.quadWidthInfo(M);
        d=BOLD.process(J.X);
	if((d!=-1)&&(d!=99)) setColors(d);
	PlotStyle.BOLD=BOLD.val;	
        if(d==99) DocumentImage.contrastInfo(M);
	d=STYLE.process(J.X);
	PlotStyle.STYLE=STYLE.val;
	if(d==99) DocumentImage.blurInfo(M);
	d=SCALE.process(J.X);
        if(d==99) DocumentImage.scaleInfo(M);
	d=AFFINE.process(J.X); 
        if(d==99) DocumentImage.fudgeInfo(M);
 	boolean dd=SC.process(e);  
        if(dd==true) DocumentImage.colorSelectorInfo(M);
	setSpecialAffine(J.X);
	setScaling(J.X);
	presetScales(J.X);
	if(EXPORT.inside(J.X)==1) M.C.JD.savePix();
	for(int i=0;i<14;++i) {
	    if(COL[i].inside(J.X)==1) RenderJulia.COL[i]=SC.C;
	}
	repaint();
	M.repaint();
    }

    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {
	requestFocus();
    }

    public void mouseExited(MouseEvent e) {}   
    public void mouseMoved(MouseEvent e) {}    
    public void mouseDragged(MouseEvent e) {}
    public void keyPressed(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}

}

